"""Summarize the analysis"""

import logging
from pathlib import Path

from fluffy.slurm_api import SlurmAPI
from fluffy.commands.multiqc import get_multiqc_cmd

from fluffy.singularity_cmd import singularity_base

LOG = logging.getLogger(__name__)


def get_summarize_cmd(
    singularity: str, out_dir: Path, sample_sheet: str, zscore: str, mincnv: str
) -> str:
    """Return a string with the command to summarize a run"""
    outfile = out_dir / "summary.csv"
    summary_cmd = (
        f"{singularity} python /bin/FluFFyPipe/scripts/generate_csv.py "
        f"--folder {str(out_dir)} --samplesheet {sample_sheet} --Zscore {zscore} --minCNV {mincnv} "
        f"> {str(outfile)}"
    )
    return summary_cmd


def summarize_workflow(
    configs: dict, afterok: list, slurm_api: SlurmAPI, dry_run: bool = False,
) -> int:
    """Run the workflow to summarize an analysis"""
    LOG.info("Run the summarize workflow")
    out_dir = configs["out"]

    singularity=singularity_base(configs["singularity"], configs["out"], configs["project"], configs["singularity_bind"])

    summarize_cmd = get_summarize_cmd(
        singularity=singularity,
        out_dir=out_dir,
        sample_sheet=configs["sample_sheet"],
        zscore=configs["summary"]["zscore"],
        mincnv=configs["summary"]["mincnv"],
    )

    multiqc_cmd=get_multiqc_cmd(singularity=singularity,input_dir=out_dir,out_dir=out_dir)

    jobid = slurm_api.run_job(
        name=f"summarize_batch",
        command=f"{multiqc_cmd}\n{summarize_cmd}",
        afterok=afterok,
        dry_run=dry_run,
    )

    return jobid



"""Workflows to run AMYCNE"""

import logging

from fluffy.commands.pipelines import amycne_ffy
from fluffy.slurm_api import SlurmAPI

LOG = logging.getLogger(__name__)


def estimate_ffy(
    configs: dict,
    sample_id: str,
    afterok: int,
    slurm_api: SlurmAPI,
    dry_run: bool = False,
) -> int:
    """Run the estimate fetal fraction with AMYCNE"""
    LOG.info("Running the estimate fetal fraction with AMYCNE workflow")
    out_dir = configs["out"]
    fetal_fraction_pipe = amycne_ffy(
        configs=configs, out_dir=out_dir, sample_id=sample_id
    )

    jobid = slurm_api.run_job(
        name=f"amycne-{sample_id}",
        command=fetal_fraction_pipe,
        afterok=[afterok],
        dry_run=dry_run,
    )

    return jobid



"""Code to check config"""

import json
import logging
import pathlib

LOG = logging.getLogger(__name__)


def file_exists(file_path: pathlib.Path) -> bool:
    """Check if a file exists.

    """
    if not file_path.exists():
        LOG.error("Could not find file %s", file_path)
        raise FileNotFoundError

    return True


def check_configs(config: dict, mkref: bool = False, skip_preface: bool = False):
    """Check if all files specified in config exists"""
    file_exists(pathlib.Path(config["reference"]))
    file_exists(pathlib.Path(config["wisecondorx"]["blacklist"]))
    file_exists(pathlib.Path(config["singularity"]))

    if mkref is False:
        file_exists(pathlib.Path(config["wisecondorx"]["reftest"]))

        if skip_preface is False:
            file_exists(pathlib.Path(config["wisecondorx"]["refpreface"]))


def get_configs(config_path: pathlib.Path) -> dict:
    """Parse and return configs"""

    with open(config_path) as config_handle:
        configs = json.load(config_handle)

    return configs



"""Commands that use the WisecondorX package"""

from pathlib import Path


def get_convert_cmd(singularity: str, out_prefix: str) -> str:
    """Get command for converting bam to called nipt"""
    cmd = (
        f"{singularity} WisecondorX convert {out_prefix}.bam "
        f"{out_prefix}.bam.wcx.npz"
    )
    return cmd


def get_newref_cmd(singularity: str, out: str, binsize: str) -> str:
    """Get command for creating reference bam to called nipt"""
    cmd = (
        f"{singularity} WisecondorX newref {out}/**/*.wcx.npz "
        f"{out.rstrip('/')}.wcxref.{binsize}.npz --nipt --binsize {binsize}"
    )
    return cmd


def get_mkref_cmd(
    singularity: str, out: str, testbinsize: str, prefacebinsize: str
) -> str:
    """Command to generate wisecondorX reference files"""
    wcx_mkref = get_newref_cmd(singularity, out, testbinsize)

    wcx_mkrefpreface = get_newref_cmd(singularity, out, prefacebinsize)

    return "\n".join([wcx_mkref, wcx_mkrefpreface])


def get_predict_cmd(
    singularity: str,
    out_prefix: Path,
    reference: str,
    blacklist: str,
    zscore: str,
    preface: bool = False,
):
    """Get a string with command for running the wisecondor X test"""
    out = out_prefix.with_suffix(".WCXpredict")
    if preface:
        out = out_prefix.with_suffix(".WCXpredict" + ".preface")

    predict_cmd = (
        f"{singularity} WisecondorX --loglevel info predict "
        f"{str(out_prefix)}.bam.wcx.npz {reference} {str(out)} --bed --blacklist "
        f"{blacklist} --zscore {zscore}"
    )

    return predict_cmd


def get_gender_cmd(singularity: str, out_prefix: str, reference: str):
    """Get a string with command for running the wisecondor X test"""
    gender_cmd = (
        f"{singularity} WisecondorX gender {out_prefix}.bam.wcx.npz "
        f"{reference} > {out_prefix}.wcx.npz.gender.txt"
    )

    return gender_cmd



"""code to generate the singularity base command"""
import logging
from pathlib import Path

def singularity_base(singularity_exe: str, out_dir: Path, project_dir: Path, singularity_bind: list):
	bind=f" --bind {str(out_dir)} --bind {str(project_dir)}"
	if singularity_bind:
		for d in singularity_bind:
			bind+= f" --bind {d}"

	singularity_base_cmd= f"singularity exec {bind} {singularity_exe}"
	return singularity_base_cmd




"""Commands that use multiqc"""
from pathlib import Path

def get_multiqc_cmd(singularity: str, input_dir: Path, out_dir: Path) -> str:
    """Get command for converting bam to called nipt"""
    cmd = (
	f"{singularity} multiqc {str(input_dir)} --outdir {str(out_dir)}"
    )
    return cmd



"""CLI to run the analysis"""
import logging

import click
import shutil

from fluffy.config import check_configs
from fluffy.deliverables import print_deliverables
from fluffy.status import print_status
from fluffy.workflows.analyse_samples import analyse_workflow


LOG = logging.getLogger(__name__)


@click.command()
@click.pass_context
@click.option(
    "--skip-preface", is_flag=True, help="Skip preface fetal fraction estimation"
)
@click.option("--dry-run", is_flag=True, help="Do not create any files")
def analyse(ctx, skip_preface, dry_run):
    """Run the pipeline to call NIPT"""
    LOG.info("Running fluffy analyse")
    samples = ctx.obj["samples"]
    configs = ctx.obj["configs"]
    configs["sample_sheet"] = ctx.obj["sample_sheet"]
    project_dir=ctx.obj["project"]
    slurm_api = ctx.obj["slurm_api"]

    config_path=configs["out"] / configs["name"]
    if config_path.exists():
        LOG.warning("Config already exists, create new dir or remove config")
        raise click.Abort

    LOG.info("Copy config to %s", config_path)
    shutil.copy(configs["config_path"], str(config_path))

    try:
        check_configs(configs, skip_preface=skip_preface)
    except FileNotFoundError as err:
        raise click.Abort

    print_deliverables(
        output_dir=configs["out"],
        project_dir=project_dir,
        samples=samples,
    )

    print_status(
        output_dir=configs["out"],
    )

    analyse_workflow(
        samples=list(samples),
        configs=configs,
        skip_preface=skip_preface,
        slurm_api=slurm_api,
        dry_run=dry_run,
    )



"""Code to get commands for running the preface program"""

from pathlib import Path

from .utils import get_outprefix


def get_preface_predict_cmd(
    singularity: str, out_dir: Path, model_dir: str, sample_id: str
) -> str:
    """Create command for doing fetal fraction estimation using Preface"""
    out_prefix = get_outprefix(out_dir, sample_id)
    predict_cmd = (
        f"{singularity} Rscript /bin/PREFACE-0.1.1/PREFACE.R predict --infile"
        f" {str(out_prefix)}.WCXpredict.preface_bins.bed --model {model_dir}/model.RData > "
        f"{str(out_prefix)}_bins.bed.PREFACE.txt"
    )

    return predict_cmd


def get_preface_model_cmd(
    singularity: str, config_path: Path, model_dir: str, settings: str,
) -> str:
    """Create command for doing fetal fraction estimation using Preface"""
    model_cmd = (
        f"{singularity} Rscript /bin/PREFACE-0.1.1/PREFACE.R train --config "
        f"{str(config_path)} --outdir {model_dir} {settings}"
    )

    return model_cmd



"""
fluffy.__main__
~~~~~~~~~~~~~~~~~~~~~

The main entry point for the command line interface.

Invoke as ``fluffy`` (if installed)
or ``python -m fluffy`` (no install required).
"""
import sys

from fluffy.cli.base import base_command

if __name__ == "__main__":
    # exit using whatever exit code the CLI returned
    sys.exit(base_command())



"""Fluffy base command"""

import logging
import pathlib
import shutil

import click
import coloredlogs

from fluffy.cli.make_analysis import analyse
from fluffy.cli.make_reference import reference
from fluffy.cli.make_rerun import rerun
from fluffy.config import get_configs
from fluffy.samplesheet import read_samplesheet
from fluffy.slurm_api import SlurmAPI
from fluffy.version import __version__

LOG = logging.getLogger(__name__)
LOG_LEVELS = ["DEBUG", "INFO", "WARNING"]


@click.group()
@click.option(
    "--log-level",
    default="INFO",
    type=click.Choice(LOG_LEVELS),
    help="Choose what log messages to show",
)
@click.option(
    "--config",
    "-c",
    required=True,
    type=click.Path(exists=True),
    help="json config file",
)
@click.version_option(__version__)
@click.option("--out", "-o", required=True, help="output folder")
@click.option(
    "--sample",
    "-s",
    type=click.Path(exists=True),
    required=True,
    help="path to samplesheet",
)
@click.option(
    "--project",
    "-p",
    type=click.Path(exists=True),
    required=True,
    help="input project folder",
)
@click.pass_context
def base_command(ctx, log_level, config, out, sample, project):
    """Fluffy base command"""
    coloredlogs.install(log_level)
    ctx.obj = {}

    out = pathlib.Path(out)
    LOG.info("Create outdir %s (if not exist)", out)
    out.mkdir(parents=True, exist_ok=True)

    config = pathlib.Path(config)
    configs = get_configs(config)
    configs["out"] = out
    configs["name"]=config.name
    configs["config_path"]=config
    ctx.obj["configs"] = configs

    new_config = out / config.name

    project_dir = pathlib.Path(project)
    ctx.obj["project"] = project_dir
    configs["project"] = project_dir

    sacct_dir = out / "sacct"
    sacct_dir.mkdir(parents=True, exist_ok=True)

    with open(sample, "r") as samplesheet:
        ctx.obj["samples"] = list(read_samplesheet(samplesheet, project_dir))

    ctx.obj["sample_sheet"] = sample

    ctx.obj["slurm_api"] = SlurmAPI(
        slurm_settings=configs["slurm"], out_dir=out,
    )


base_command.add_command(reference)
base_command.add_command(rerun)
base_command.add_command(analyse)
